﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.Net;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;

namespace DemoThinkGeoContrast
{
    public class AdjustableWmsRasterSource : WmsRasterSource
    {
        private byte contrast = 128;
        private byte brightness = 128;
        private RectangleShape previousworldexent = new RectangleShape();
        private Bitmap previousBitmap;
        public byte Contrast { get { return this.contrast; } set { this.contrast = value; } }
        public byte Brightness { get { return this.brightness; } set { this.brightness = value; } }


        public AdjustableWmsRasterSource()
        {
                
        }
        public AdjustableWmsRasterSource(Uri uri)
            : base(uri)
        {
            
        }
        public AdjustableWmsRasterSource(Uri uri, WebProxy proxy)
            : base(uri, proxy)
        {

        }

        protected override GeoImage GetImageCore(RectangleShape worldExtent, int canvasWidth, int canvasHeight)
        {

            GeoImage originalGeoImage;
            Bitmap orgGeoBitmap;
            
            if (previousworldexent.UpperLeftPoint.X == worldExtent.UpperLeftPoint.X && previousworldexent.UpperLeftPoint.Y == worldExtent.UpperLeftPoint.Y  && previousworldexent.LowerRightPoint.X == worldExtent.LowerRightPoint.X && previousworldexent.LowerRightPoint.Y == worldExtent.LowerRightPoint.Y)
            {
                orgGeoBitmap = previousBitmap;
            }
            else
            {
                originalGeoImage = base.GetImageCore(worldExtent, canvasWidth, canvasHeight);
                Stream geoImgStream = originalGeoImage.GetImageStream(new GdiPlusGeoCanvas());
                orgGeoBitmap = (Bitmap)Bitmap.FromStream(geoImgStream);
            }
            previousBitmap = orgGeoBitmap;
            previousworldexent = worldExtent;
                
            Bitmap contrastedGeoBitmap = ApplyImageTransform(orgGeoBitmap, this.Brightness, this.Contrast);   // Method that adjusts the contrast of a bitmap

            MemoryStream contrastedImgStream = new MemoryStream();
            contrastedGeoBitmap.Save(contrastedImgStream, System.Drawing.Imaging.ImageFormat.Tiff);

            GeoImage geoimg = new GeoImage(contrastedImgStream);
            return geoimg;
                           
        }

        private static Bitmap ApplyImageTransform(Bitmap Image, byte brightness, byte contrast)
        {
            Bitmap NewBitmap = (Bitmap)Image.Clone();
            BitmapData data = NewBitmap.LockBits(
                new Rectangle(0, 0, NewBitmap.Width, NewBitmap.Height),
                ImageLockMode.ReadWrite,
                PixelFormat.Format32bppArgb);

            // calculate slope
            // map the contrast to a slope value 0-256 -> 0-pi/2 -> 0-(slightly less than infinity)
            double slope = Math.Tan((double)contrast * Math.PI / 512D);

            unsafe
            {
                for (int y = 0; y < NewBitmap.Height; ++y)
                {
                    byte* row = (byte*)data.Scan0 + (y * data.Stride);
                    int columnOffset = 0;
                    for (int x = 0; x < NewBitmap.Width; ++x)
                    {
                        byte B = row[columnOffset];
                        byte G = row[columnOffset + 1];
                        byte R = row[columnOffset + 2];

                        // Apply brightness/contrast
                        byte bcB = (byte)Math.Max(Math.Min(slope * (B - 128) + brightness, 255), 0);
                        byte bcG = (byte)Math.Max(Math.Min(slope * (G - 128) + brightness, 255), 0);
                        byte bcR = (byte)Math.Max(Math.Min(slope * (R - 128) + brightness, 255), 0);

                        row[columnOffset] = bcB;
                        row[columnOffset + 1] = bcG;
                        row[columnOffset + 2] = bcR;

                        columnOffset += 4;
                    }
                }
            }

            NewBitmap.UnlockBits(data);

            return NewBitmap;
        }
    }
}
