﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace DemoThinkGeoContrast
{
    public partial class Form1 : Form
    {
        AdjustableWmsLayerSource wms;
        LayerOverlay overlay;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.DeepOcean);

            string wmsLoc = "http://datamil.delaware.gov/wmsconnector/com.esri.wms.Esrimap/DE_aerial02";
            wms = new AdjustableWmsLayerSource(new Uri(wmsLoc));
            AdjustableWmsRasterSource wmsSource = (AdjustableWmsRasterSource)wms.ImageSource;

            wmsSource.SendingRequest += wmsSource_SendingRequest;
            wmsSource.SentRequest += this.wmsSource_SentRequest;
            wms.Open();

            wms.OutputFormat = "IMAGE/PNG";
            wms.Crs = "EPSG:4326";
            
            wms.ActiveLayerNames.Add(wms.GetServerLayerNames()[0]);

            overlay = new LayerOverlay();
            overlay.Layers.Add(wms);
            this.winformsMap1.Overlays.Add(overlay);

            
            FileBitmapTileCache cache = new FileBitmapTileCache("cache", "wms");
            overlay.TileCache = cache;
            overlay.TileCache.ClearCache();

            this.winformsMap1.CurrentExtent = wms.GetBoundingBox();
            this.winformsMap1.Refresh();
        }

        // A new request is sent when refresh() is called rather than using the cached image
        void wmsSource_SentRequest(object sender, SentRequestWmsRasterSourceEventArgs e)
        {
            Console.WriteLine("Sent request {0}", DateTime.Now);
        }

        // A new request is sent when refresh() is called rather than using the cached image
        void wmsSource_SendingRequest(object sender, SendingRequestWmsRasterSourceEventArgs e)
        {
            Console.WriteLine("Sending request {0}", DateTime.Now);
        }

        private void Brightness_TrackBar_Scroll(object sender, EventArgs e)
        {
            
            wms.Brightness = (byte)this.Brightness_TrackBar.Value;
            this.winformsMap1.Refresh(overlay);     // This is my problem
        }

        private void Contrast_TrackBar_Scroll(object sender, EventArgs e)
        {
            wms.Contrast = (byte)this.Contrast_TrackBar.Value;
            this.winformsMap1.Refresh(overlay);     // This is my problem
        }
    }
}
